procedure omflat(infile)

#summary of changes in Sep2018
#
#In flatcor = "slitlet" and illcorr=slitlet option, the normalization was always done 
#  on slitlet=1 even when it is circular. The first rectangular slitlet is now used
#new flatcor = "slitlet"  illumination correction options:
#illcorr = none : final_flat_none =  f_i/median_flat_image
#illcorr = relative or slitlet - normalize wrt the first rectangular slit: 
#	final_flat_relative =  (f_i/median_flat_image) * (median_value_i/median_value_first_slit)
#illcorr = illum - use illumination : final_flat_illum =  f_ill_funcion*final_flat_relative
#
# Description of default options
#For longslit spectra, divide the input by its median image. Multiply this by 
#the illumination function (_ill output image) as calculated by IRAF/illumination 
#and save it as _norm output image.
#
#For MOS espectra, divide the input by its median image for each slitlet (step1). 
#Then correct the relative "efficiencies" of slitlets by normalizing them to the 
#"normslit" flux (step2). Then, obtain the median of 51 pixels on the wavelength 
#axis around the slitlet position to obtain the slitlet function (step3). Repeat 
#this slitlet function all along the wavelength axis and save it as _ill output (step4). 
#Multiply, the slit function image to the normalized flat from step2 to obtain the 
# _norm output image.

string  infile = "gtc14aob4_object_sum_wl_tlt"      {prompt="Input 2D file template"}
string  outfile = "DEFAULT"    	   {prompt="Output Master flat [infile//_norm and _ill]"}
#string  flatcor = "slitlet"   	   {prompt="Flat correction method [slitlet/longslit]"}
string  illcorr = "none"   	   {prompt="Illumination correction method [none, illum, slitlet]"}
string  normslit= "longest"   	   {prompt="Normalize fluxes to this slitlet [first, longest, brightest, faintest, center, none]"}
real    yillcen = 0.0              {prompt="Center Y value of illumination correction bin [0=slityx]"}
real    yillwid = 20.0             {prompt="Width of illumination correction bin"}
int     xwin    = "3"              {prompt="X window size of median filter"}
int     ywin    = "3"              {prompt="Y window size of median filter"}
string	medbound= "nearest"	   {prompt="Boundary (constant,nearest,reflect,wrap) for median"}
bool	illinte = no		   {prompt="Enter illumin interactively?"}
string  illfunc = "spline3"	   {prompt="Fitting function in Illumination"}
int	illorde = 11	   	   {prompt="Fitting function order in Illumination"}
int	illnbin = 5   	   	   {prompt="Number of fitting bins in Illumination"}
bool    del     = yes              {prompt="Delete all intermediate files?"}

begin
	string  filin, filen, outnam, filinp, function, sec[999], stype, s1, s2
	string	boundary, statsec[999], illtype, fcor, ntype, logfile, statsecn
	string	styp[999], size
        real    slitx, slity, slityx, slito, flatnor, flat[999]
	real	flat_max, flat_min, ycen, ywid
	int     xw, yw, i1, i2, order, nbin, nmode, first_slit, iref, dx
        int     nslits, ix1, ix2, islit, nslice, is, is1, is2, isl
	int	ilongest, imax, imin, icenter, nx, ny
        bool    slice[999], delete, cor_ill, inter

        filinp = infile; xw = xwin; yw = ywin; outnam = outfile; delete = del
	nbin = illnbin; order = illorde; function = illfunc
	illtype = illcorr; inter = illinte; boundary = medbound
	ycen = yillcen; ywid = yillwid 
	#fcor = flatcor 
	ntype = normslit
	logfile = "omflat.log"

	#if (substr(fcor, 1, 1) == "l"){fcor= "longslit"} else {fcor= "slitlet"}
	#if (substr(illtype, 1, 1) == "n"){illtype = "none"} 
        #else if (substr(illtype, 1, 1) == "i"){illtype = "illum"} else {illtype = "slitlet"}
	#print(fcor, " ",illtype)

	i2 = strstr(".",filinp)
	if (i2 > 0) {i2 = i2 -1} else {i2 = strlen(filinp)}
        i1 = strstr("[",filinp)
	if (i1 == 0){filin = substr(filinp, 1, i2)}
        if (i1 > 0) {i2 = strstr("]",filinp); filin = substr(filinp, 1, i1-1)}

	if (strupr(outnam) == "DEFAULT") {outnam = filin//"_norm"}
    	imdelete (outnam, veri=no,  >& "dev$null")
    	imdelete (filin//"_ill", veri=no,  >& "dev$null")
    	imdelete (filin//"_normill", veri=no,  >& "dev$null")

        omslits (filin, verb-)
        nslits = omslits.nslitlets
        list = omslits.out
	nmode = omslits.obsmod

	#if (nmode == 1 && illtype != "none"){illtype = "slitlet"}
	#print(nmode, " ",illtype)
	#if (nmode > 1 && illtype != "slitlet") {
	fcor = "slitlet"
	if (nmode > 1){fcor = "longslit"}
	#print (fcor, " ", illtype)
	if (fcor == "longslit") {
    	imdelete (filin//"_med", veri=no, >& "dev$null")
    	imdelete (filin//"_nor", veri=no,  >& "dev$null")

        #--------------------------------------------------------
	#print ("Step 1: Running a median filter to the image")
        #--------------------------------------------------------

	median (filin, filin//"_med", xw=xw, yw=yw, boundar=boundary, verb-)

        #--------------------------------------------------------
	#print ("Step 2: Normalization --- original/median ")
        #--------------------------------------------------------

	imarith (filin, "/", filin//"_med", filin//"_nor", verb=no)

        #--------------------------------------------------------
	#print ("Step 3: Illumination correction")
        #--------------------------------------------------------
        if (illtype == "illum") {
    	imdelete (filin//"_ill", veri=no,  >& "dev$null")
	illum (filin//"_med", filin//"_ill", functio=function, order=order, interac=inter, bins=" ", nbins=nbin)
        #--------------------------------------------------------
	#print ("Step 4: Master Flat = FlatNorm*FlatIllum")
        #--------------------------------------------------------

    	imdelete (filin//"_flt", veri=no,  >& "dev$null")
	imarith (filin//"_nor", "*", filin//"_ill", filin//"_flt", verb=no)
        imcopy (filin//"_flt", outnam, ver-)
	} else {imcopy (filin//"_nor", outnam, ver-)}
	if (delete) {
    	imdelete (filin//"_med", veri=no, >& "dev$null")
    	imdelete (filin//"_nor", veri=no,  >& "dev$null")
    	imdelete (filin//"_ill", veri=no,  >& "dev$null")
    	imdelete (filin//"_flt", veri=no,  >& "dev$null")
    	#imdelete (filin//"_flt_ill", veri=no,  >& "dev$null")
	}
	}

	if (fcor == "slitlet") {
	imexpr ("a*0", outnam, filinp, verb-)
	imexpr ("a*0", filin//"_ill", filinp, verb-)
	imexpr ("a*0", filin//"_normill", filinp, verb-)
        i1 = 0
	first_slit = 0; dx = 1; flat_max = 0.01; flat_min = 1.0E9

        for (i = 1; i <= nslits; i = i + 1) {
        i1 = fscan(list, islit, ix1, ix2, slitx, slity, slityx, stype, slito)
	styp[i] = stype
        if (stype != "circular") {

        sec[i] = "["//str(ix1)//":"//str(ix2)//",*]"
        imdelete (filin//"_"//str(i), ver-, >& "dev$null")
        imcopy (filin//sec[i], filin//"_"//str(i), ver-)

	if (first_slit == 0){first_slit = i}
	if ((ix2 - ix1 + 1) > dx){dx = ix2 - ix1 +1; ilongest = i} 
	if (ix1 < 1000){icenter = i} 
	if (ycen > ywid) {slityx = ycen}
	#statsec[i] = "["//str(ix1)//":"//str(ix2)//","//str(int(slityx-20.0))//":"//str(int(slityx+20.0))//"]"
	statsec[i] = "["//str(ix1)//":"//str(ix2)//","//str(int(slityx-ywid))//":"//str(int(slityx+ywid))//"]"
        imstat (filin//statsec[i], fields="midpt", format-) | scan (flat[i])
	if (flat[i] > flat_max) {flat_max = flat[i]; imax = i}
	if (flat[i] < flat_min) {flat_min = flat[i]; imin = i}
	}}

	iref = 0
	#if (ntype == "none"){iref = 0}
	if (ntype == "first"){iref = first_slit}
	if (ntype == "longest"){iref = ilongest}
	if (ntype == "brightest"){iref = imax}
	if (ntype == "faintest"){iref = imin}
	if (ntype == "center"){iref = icenter}
	print("------------------ ", filin, >> logfile)
	print("First slitlet no = ", first_slit, >> logfile)
	print("The longest slitlet no = ", ilongest, " Length = ", dx, >> logfile)
	print("The brightest slitlet no = ", imax, " Counts = ", flat_max, >> logfile)
	print("The faintest slitlet no = ", imin, " Counts = ", flat_min, >> logfile)
	print("The slitlet no closest to the canonical position [x=1000] = ", icenter, " Counts =", flat[icenter], >> logfile)
	print("Normalization method : ", ntype, ". Slitlet no = ", iref, >> logfile)

	#filen = filin//"_"//str(iref)
    	#imdelete (filen//"_med", veri=no, >& "dev$null")
    	#imdelete (filen//"_nor", veri=no,  >& "dev$null")

	#median (filen, filen//"_med", xw=xw, yw=yw, boundar=boundary, verb-)
	#imarith (filen, "/", filen//"_med", filen//"_nor", verb=no)
	statsecn = "[*,"//str(int(slityx-20.0))//":"//str(int(slityx+20.0))//"]"
        #imstat (filen//"_nor"//statsecn, fields="midpt", format-) | scan (flat[iref])
	if (iref > 0){print("Normalization value: ", flat[iref], >> logfile)}

    	delete ("imcombine.log", veri=no, >& "dev$null")
        for (i = 1; i <= nslits; i = i + 1) {
        if (styp[i] != "circular") {

	filen = filin//"_"//str(i)
    	imdelete (filen//"_med", veri=no, >& "dev$null")
    	imdelete (filen//"_nor", veri=no,  >& "dev$null")
    	imdelete (filen//"_flt", veri=no,  >& "dev$null")
    	imdelete (filen//"_ill1d", veri=no,  >& "dev$null")
    	imdelete (filen//"_ill", veri=no,  >& "dev$null")
    	imdelete (filen//"_flt_ill", veri=no,  >& "dev$null")

        #--------------------------------------------------------
	#print ("Step 1: Running a median filter to the image")
        #--------------------------------------------------------

	median (filen, filen//"_med", xw=xw, yw=yw, boundar=boundary, verb-)

        #--------------------------------------------------------
	#print ("Step 2: Normalization --- original/median ")
        #--------------------------------------------------------

	imarith (filen, "/", filen//"_med", filen//"_nor", verb=no)

        if (illtype == "none") {
	imstat (filen//"_nor", format-, >> logfile)
		imcopy (filen//"_nor", outnam//sec[i], ver-)
	} else {
        #if (illtype == "slitlet") {
        #--------------------------------------------------------
	#print ("Step 3: Illumination correction")
        #--------------------------------------------------------
        if (iref > 0){flatnor = flat[i]/flat[iref]} else {flatnor = 1.0}
	#print(filin//statsec[i], " ", flat[i], flatnor, i, >> "temp1.dat")
	print(filin//statsec[i], " ", flat[i], flatnor, i, >> logfile)
	imarith (filen//"_nor", "*", flatnor, filen//"_flt", verb=no)
        imcopy (filen//"_flt", outnam//sec[i], ver-)
	if (illtype == "slitlet") {
	hselect (filen//"_flt", "NAXIS1 NAXIS2", yes) | scan(nx, ny)
	size = str(nx)//","//str(ny)
	imcombine (filen//"_flt"//statsecn, filen//"_ill1d", combine="median", logfile="imcombine.log", project+) 
	#imcombine (filen//"_flt"//statsecn, filen//"_ill", headers="", bpmasks="", 
	#rejmasks="", nrejmasks="", expmasks="", sigmas="", imcmb="$I", logfile=logfile, 
	#combine="median", reject="none", project=yes, outtype="real", outlimits="", 
	#offsets="none", masktype="none", maskvalue="0", blank=0., scale="none", 
	#zero="none", weight="none", statsec="", expname="", lthreshold=INDEF, 
	#hthreshold=INDEF, nlow=1, nhigh=1, nkeep=1, mclip=yes, lsigma=3., hsigma=3., 
	#rdnoise="0.", gain="1.", snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.)
	imexpr ("a", filen//"_ill", filen//"_ill1d", dims=size, verb-)
        imcopy (filen//"_ill", filin//"_ill"//sec[i], ver-)
	imarith (filen//"_flt", "*", filen//"_ill", filen//"_flt_ill", verb=no)
        imcopy (filen//"_flt_ill", filin//"_normill"//sec[i], ver-)
	}

	if (illtype == "illum") {
	#illum (filen//"_med", filen//"_ill", functio=function, order=order, interac=inter, bins=" ", nbins=nbin)
	illum (filen//"_flt", filen//"_ill", functio=function, order=order, interac=inter, bins=" ", nbins=nbin)
	#print ("Step 4: Master Flat = FlatNorm*FlatIllum")
	#imarith (filen//"_nor", "*", filen//"_ill", filen//"_flt", verb=no)
	imarith (filen//"_flt", "*", filen//"_ill", filen//"_flt_ill", verb=no)
        imcopy (filen//"_flt_ill", outnam//"_ill"//sec[i], ver-)
	}  #if illum=yes
	}  #illtype != none

	if (delete) {
    	imdelete (filen//"_med", veri=no, >& "dev$null")
    	imdelete (filen//"_nor", veri=no,  >& "dev$null")
    	imdelete (filen//"_ill", veri=no,  >& "dev$null")
    	imdelete (filen//"_ill1d", veri=no,  >& "dev$null")
    	imdelete (filen//"_flt", veri=no,  >& "dev$null")
    	imdelete (filen//"_flt_ill", veri=no,  >& "dev$null")
	}
    	imdelete (filen, veri=no, >& "dev$null")
        #imdelete (filin//"_"//str(i), ver-, >& "dev$null")
        }   #!= "circular"
        }   #i = 1; i <= nslits
	}   #fcor = slitlet
	if (illtype == "none") {print (" Normalized flat stored in ", outnam)} 
	else {print (" Normalized and illumination corrected flat stored in ", outnam)}
	#print (" *** Done *** ")
	hedit (outnam, "OMFLAT", "yes", add+, ver-, show-)
	hedit (outnam, "FLATCOR", fcor, add+, ver-, show-)
	hedit (outnam, "ILLCOR", illtype, add+, ver-, show-)
	hedit (outnam, "NORMTYPE", ntype, add+, ver-, show-)
	hedit (outnam, "NORMSLIT", iref, add+, ver-, show-)
	if (imaccess(filin//"_ill")){
	hedit (filin//"_ill", "OMFLAT", "yes", add+, ver-, show-)
	hedit (filin//"_ill", "FLATCOR", fcor, add+, ver-, show-)
	hedit (filin//"_ill", "ILLCOR", illtype, add+, ver-, show-)
	hedit (filin//"_ill", "NORMTYPE", ntype, add+, ver-, show-)
	hedit (filin//"_ill", "NORMSLIT", iref, add+, ver-, show-)
	}
	if (imaccess(filin//"_normill")){
	hedit (filin//"_normill", "OMFLAT", "yes", add+, ver-, show-)
	hedit (filin//"_normill", "FLATCOR", fcor, add+, ver-, show-)
	hedit (filin//"_normill", "ILLCOR", illtype, add+, ver-, show-)
	hedit (filin//"_normill", "NORMTYPE", ntype, add+, ver-, show-)
	hedit (filin//"_normill", "NORMSLIT", iref, add+, ver-, show-)
	}

end

